@@ -1,17 +1,31 @@
 #!/usr/bin/perl -I../lib
 #
-# Copyright (c) 2005 Messiah College. This program is free software.
-# You can redistribute it and/or modify it under the terms of the
-# GNU Public License as found at http://www.fsf.org/copyleft/gpl.html.
+# This file is part of DKIMproxy, an SMTP-proxy implementing DKIM.
+# Copyright (c) 2005-2008 Messiah College.
+# Written by Jason Long <jlong@messiah.edu>.
+#
+# This program is free software; you can redistribute it and/or
+# modify it under the terms of the GNU General Public License
+# as published by the Free Software Foundation; either version 2
+# of the License, or (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor,
+# Boston, MA  02110-1301, USA.
 #
-# Written by Jason Long, jlong@messiah.edu.
 
 use strict;
 use warnings;
 use IO::File;
 use MIME::Entity;
 
-use Mail::DKIM 0.29;
+use Mail::DKIM 0.34;
 use Mail::DKIM::Verifier;
 
 use constant FROM_ADDR => 'admin@dkimtest.jason.long.name';
@@ -35,7 +49,7 @@ my $attach_original_msg;
 
 # read message from stdin, catching from address and subject
 my @message_lines;
-my $canonicalized = "";
+my %canonicalized;
 while (<STDIN>)
 {
 	s/\n\z/\015\012/;
@@ -63,14 +77,12 @@ $fh->seek(0, 0);
 my $result;
 my $result_detail;
 my @signatures;
-my $a_policy;
-my $a_policy_result;
-my $s_policy;
-my $s_policy_result;
+my @policies;
+my @policy_results;
 eval
 {
 	my $dkim = Mail::DKIM::Verifier->new(
-			Debug_Canonicalization => \$canonicalized,
+			Debug_Canonicalization => \&debug_canonicalization,
 		);
 	$dkim->load($fh);
 
@@ -83,11 +95,8 @@ eval
 	}
 
 	@signatures = $dkim->signatures;
-
-	$a_policy = $dkim->fetch_author_policy;
-	$a_policy_result = $a_policy->apply($dkim);
-	$s_policy = $dkim->fetch_sender_policy;
-	$s_policy_result = $s_policy->apply($dkim);
+	@policies = $dkim->policies;
+	@policy_results = map { $_->apply($dkim) } @policies;
 };
 if ($@)
 {
@@ -134,21 +143,20 @@ if (@signatures > 1)
 	$verify_results_text .= "\n";
 }
 
+my $hint_text = check_for_hints();
+
 my $policy_results_text = "";
-if ($a_policy_result && $a_policy_result ne "neutral")
+for (my $i = 0; $i < @policies; $i++)
 {
-	my $location = $a_policy->location;
-	$policy_results_text =
-"This is the result after checking the DKIM policy at \"$location\":
-  $a_policy_result
-\n";
-}
-if ($s_policy_result && $s_policy_result ne "neutral")
-{
-	my $location = $s_policy->location;
-	$policy_results_text =
-"This is the result after checking the DomainKeys policy at \"$location\":
-  $s_policy_result
+	my $policy = $policies[$i];
+	my $policy_result = $policy_results[$i];
+
+	next unless $policy_result && $policy_result ne "neutral";
+	my $location = $policy->location;
+	my $policy_type = $policy->name;
+	$policy_results_text .=
+"This is the result after checking the $policy_type policy at \"$location\":
+  $policy_result
 \n";
 }
 
@@ -172,7 +180,10 @@ subject.
 }
 
 # part one, literal text containing result of test
-my $PRODUCT = "Mail::DKIM " . $Mail::DKIM::VERSION;
+my $PRODUCT = "Mail::DKIM";
+my $VERSION = $Mail::DKIM::VERSION;
+my $PRODUCT_URL = "http://dkimproxy.sourceforge.net/";
+
 $top->attach(
 	Type => "text/plain",
 	Data => [
@@ -184,9 +195,10 @@ $top->attach(
 		"auto-responder looks for ANY passing signature, including DomainKeys\n",
 		"signatures.\n",
 		"\n",
+		$hint_text,
 		"Thank you for using the dkimproxy DKIM Auto Responder.\n",
-		"This Auto Responder tests the verification routines of $PRODUCT.\n",
-		"For more information about Mail::DKIM, see http://jason.long.name/dkimproxy/\n",
+		"This Auto Responder tests the verification routines of $PRODUCT $VERSION.\n",
+		"For more information about $PRODUCT, see $PRODUCT_URL\n",
 		"\n",
 		"If you have any questions about this automated tester, or if you\n",
 		"received this message in error, please send a note to\n",
@@ -205,18 +217,21 @@ if ($attach_original_msg)
 		Disposition => "attachment",
 		Data => \@message_lines);
 }
-if ($attach_original_msg && length($canonicalized))
+if ($attach_original_msg)
 {
 	# part three, canonicalized message
 	# FIXME - by attaching it as text/plain, the linefeed characters
 	# are subject to conversion during the encoding/decoding process.
 	# It may be better to attach as a binary object?
-	$top->attach(
-		Type => "application/octet-stream",
-		Encoding => "base64",
-		Filename => "canonicalized.txt",
-		Disposition => "attachment",
-		Data => $canonicalized);
+	foreach my $canonicalized (values %canonicalized)
+	{
+		$top->attach(
+			Type => "application/octet-stream",
+			Encoding => "base64",
+			Filename => "canonicalized.txt",
+			Disposition => "attachment",
+			Data => $canonicalized->{text});
+	}
 }
 
 # send it
@@ -231,9 +246,40 @@ sub make_auth_result
 
 	my $type = $signature->isa("Mail::DKIM::DkSignature")
 		? "domainkeys" : "dkim";
-	my $tag = $signature->can("identity_tag")
-		? $signature->identity_tag : "i";
+	my $tag = $signature->can("identity_source")
+		? $signature->identity_source : "header.i";
 
 	return "$type=" . $signature->result_detail
 		. " $tag=" . $signature->identity;
 }
+
+sub debug_canonicalization
+{
+	my ($text, $canonicalization) = @_;
+
+	$canonicalized{$canonicalization}
+		||= { text => "", canon => $canonicalization };
+	$canonicalized{$canonicalization}->{text} .= $text;
+}
+
+sub check_for_hints
+{
+	my @hints;
+
+	if ($result_detail =~ /body has been altered/)
+	{
+		if (grep /^\./, @message_lines)
+		{
+			push @hints, "Your message contains lines beginning with a period, so check that\n   your implementation signs before dot stuffing.";
+		}
+	}
+
+	if (@hints)
+	{
+		return "*** WHY DID MY MESSAGE FAIL? ***\n"
+		. "Looking at your specific message, this auto-responder suggests\n"
+		. "checking the following:\n"
+		. join("", map " * $_\n", @hints). "\n";
+	}
+	return "";
+}
